 /***************************************************************************\
|*                                                                           *|
|* Copyright © 1989-1994 by Artificial Horizons®, Inc.  All rights reserved. *|
|*                                                                           *|
|*     This material  constitutes  the trade  secrets  and confidential,     *|
|*     proprietary information of Bruce Factor and Curtis Priem,  and is     *|
|*     under license  to Artificial Horizons, Inc.   The material is not     *|
|*     to be disclosed,  reproduced,  copied,  or used in any manner not     *|
|*     permitted under license from Artificial Horizons, Inc.   For more     *|
|*     information email "ahi-interest@ahi.com".                             *|
|*                                                                           *|
 \***************************************************************************/

#ifndef _atl_h
#define _atl_h


#define	ATL_COLOR4	(0x00000001)
#define	ATL_COLOR8	(0x00000002)
#define	ATL_COLOR12	(0x00000004)
#define	ATL_COLOR15	(0x00000008)
#define	ATL_COLOR16	(0x00000010)
#define	ATL_COLOR24	(0x00000020)
#define	ATL_COLOR32	(0x00000040)
#define	ATL_BLUE_WATER	(0x00000100)



typedef struct atl_color_struct Atl_color;
struct atl_color_struct { 
  unsigned char alpha;
  unsigned char blue;
  unsigned char green;
  unsigned char red;
};

#if defined (__cplusplus)
extern "C" {
#endif

extern int atl_color(double long_00, double lat_00,double long_0c, double lat_0c,
						double long_r0, double lat_r0,double long_rc, double lat_rc,
						char* filename,int num_rows, int num_cols,double gamma);


extern int atl_color_post(double long_00, double lat_00,double long_0c, double lat_0c,
						  double long_r0, double lat_r0,double long_rc, double lat_rc,
						  double gamma,int flags );

extern int atl_elevation(double long_00, double lat_00,double long_0c, double lat_0c,
							double long_r0, double lat_r0,double long_rc, double lat_rc,
							char* filename,int num_rows,int num_cols);

extern int atl_elevation_post(double long_00, double lat_00, double long_0c, double lat_0c,
								double long_r0, double lat_r0,double long_rc, double lat_rc);

extern int atl_pclevels(double long_00, double lat_00, double long_0c, double lat_0c,
                         double long_r0, double lat_r0, double long_rc, double lat_rc);

extern int atl_rasterfile(double long_00, double lat_00, double long_0c, double lat_0c,
							double long_r0, double lat_r0, double long_rc, double lat_rc,
							char *fn_ras, int flags);

extern int atl_shading(double long_00, double lat_00, double long_0c, double lat_0c,
						double long_r0, double lat_r0,double long_rc, double lat_rc,
						Atl_color color[2],double time_of_day);

#if defined (__cplusplus)
};	//extern "C" {
#endif


#endif //!_atl_h
